@@ -167,19 +167,6 @@ node_find(node_t* node, const gchar* filename, gboolean create_on_missing)
     return child;
 }
 
-node_t*
-node_find_accessible_ancestor(node_t* node)
-{
-    for (node = NODE_PARENT(node); node != ROOT; node = NODE_PARENT(node)) {
-        if (NODE_HAS_STATE(node, NODE_STATE_ASSOCIATED) || node_lstat(node) == 0) {
-            return node;
-        }
-        /* else it isn't existing or not accessible */
-    }
-    g_assert(node);
-    return node;
-}
-
 gint
 node_lstat(node_t *f)
 {
@@ -256,7 +243,7 @@ node_create_children_snapshot(node_t *f, gint created_event, gboolean emit)
     }
 }
 
-/**
+/*
  * If all active children nodes are ported, then cancel monitor the parent
  * node. If we know how many children are created, then we can stop accordingly.
  *
@@ -386,7 +373,7 @@ node_children_num (node_t *f)
     return g_hash_table_size (f->children);
 }
 
-/**
+/*
  * depth first delete recursively
  */
 static gboolean
@@ -407,7 +394,7 @@ node_class_init()
     return port_class_init (node_add_event);
 }
 
-/**
+/*
  * Adjust self on failing to Port
  */
 void
@@ -417,9 +404,24 @@ node_adjust_deleted(node_t* f)
 
     FN_W ("%s %s\n", __func__, NODE_NAME(f));
 
-    for (ancestor = node_find_accessible_ancestor(f);
-         !NODE_HAS_STATE(ancestor, NODE_STATE_ASSOCIATED) && port_add(ancestor) != 0;
-         ancestor = node_find_accessible_ancestor(ancestor)) { /* Empty */ }
+    for (ancestor = NODE_PARENT(f);
+         ancestor != NULL;
+         ancestor = NODE_PARENT(ancestor)) {
+        /* Stop if we find a node which been already associated or is existed
+         * and can be associated.
+         */
+        if (NODE_HAS_STATE(ancestor, NODE_STATE_ASSOCIATED) ||
+          (node_lstat(ancestor) == 0 && port_add(ancestor) == 0)) {
+            break;
+        }
+    }
+
+    /* We assume we shouldn't reach here, because Root is always existed and
+     * associated. But given bugster#6955199, if PORT FS has problems on root,
+     * we may reach here. So just return ROOT and the whole GIO fen backend will
+     * fail.
+     */
+    /* g_assert(ancestor != NULL); */
 }
 
 
@@ -479,7 +481,7 @@ node_emit_events(node_t *f, const node_event_t *ne)
     }
 }
 
-/**
+/*
  * node_add_event:
  *
  */
@@ -637,4 +639,3 @@ node_event_delete (node_event_t* ev)
 {
     g_free (ev);
 }
-
