@@ -39,6 +39,7 @@ static GMainLoop *loop = NULL;
 static void
 test_connection_life_cycle (void)
 {
+  gboolean ret;
   GDBusConnection *c;
   GDBusConnection *c2;
   GError *error;
@@ -88,9 +89,14 @@ test_connection_life_cycle (void)
   g_assert_no_error (error);
   g_assert (c2 != NULL);
   g_assert (!g_dbus_connection_is_closed (c2));
-  g_dbus_connection_close (c2);
+  ret = g_dbus_connection_close_sync (c2, NULL, &error);
+  g_assert_no_error (error);
+  g_assert (ret);
   _g_assert_signal_received (c2, "closed");
   g_assert (g_dbus_connection_is_closed (c2));
+  ret = g_dbus_connection_close_sync (c2, NULL, &error);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CLOSED);
+  g_assert (!ret);
   g_object_unref (c2);
 
   /*
@@ -409,6 +415,7 @@ test_connection_signals (void)
                                            "Foo",
                                            "/org/gtk/GDBus/ExampleInterface",
                                            NULL,
+                                           G_DBUS_SIGNAL_FLAGS_NONE,
                                            test_connection_signal_handler,
                                            &count_s1,
                                            NULL);
@@ -418,6 +425,7 @@ test_connection_signals (void)
                                            "Foo",
                                            "/org/gtk/GDBus/ExampleInterface",
                                            NULL,
+                                           G_DBUS_SIGNAL_FLAGS_NONE,
                                            test_connection_signal_handler,
                                            &count_s2,
                                            NULL);
@@ -427,6 +435,7 @@ test_connection_signals (void)
                                            "NameOwnerChanged",      /* member */
                                            "/org/freedesktop/DBus", /* path */
                                            NULL,
+                                           G_DBUS_SIGNAL_FLAGS_NONE,
                                            test_connection_signal_handler,
                                            &count_name_owner_changed,
                                            NULL);
@@ -435,14 +444,15 @@ test_connection_signals (void)
    * subscriptions instead of just 1 call to each of the N subscriptions.
    */
   s1b = g_dbus_connection_signal_subscribe (c1,
-                                           ":1.2",
-                                           "org.gtk.GDBus.ExampleInterface",
-                                           "Foo",
-                                           "/org/gtk/GDBus/ExampleInterface",
-                                           NULL,
-                                           test_connection_signal_handler,
-                                           &count_s1b,
-                                           NULL);
+                                            ":1.2",
+                                            "org.gtk.GDBus.ExampleInterface",
+                                            "Foo",
+                                            "/org/gtk/GDBus/ExampleInterface",
+                                            NULL,
+                                            G_DBUS_SIGNAL_FLAGS_NONE,
+                                            test_connection_signal_handler,
+                                            &count_s1b,
+                                            NULL);
   g_assert (s1 != 0);
   g_assert (s1b != 0);
   g_assert (s2 != 0);
@@ -615,14 +625,14 @@ test_connection_filter (void)
                                       "GetNameOwner");
   g_dbus_message_set_body (m, g_variant_new ("(s)", "org.freedesktop.DBus"));
   error = NULL;
-  g_dbus_connection_send_message (c, m, &data.serial, &error);
+  g_dbus_connection_send_message (c, m, G_DBUS_SEND_MESSAGE_FLAGS_NONE, &data.serial, &error);
   g_assert_no_error (error);
 
   while (data.num_handled == 0)
     g_thread_yield ();
 
   g_dbus_message_set_serial (m, 0);
-  g_dbus_connection_send_message (c, m, &data.serial, &error);
+  g_dbus_connection_send_message (c, m, G_DBUS_SEND_MESSAGE_FLAGS_NONE, &data.serial, &error);
   g_assert_no_error (error);
 
   while (data.num_handled == 1)
@@ -631,6 +641,7 @@ test_connection_filter (void)
   g_dbus_message_set_serial (m, 0);
   r = g_dbus_connection_send_message_with_reply_sync (c,
                                                       m,
+                                                      G_DBUS_SEND_MESSAGE_FLAGS_NONE,
                                                       -1,
                                                       &data.serial,
                                                       NULL, /* GCancellable */
@@ -645,6 +656,7 @@ test_connection_filter (void)
   g_dbus_message_set_serial (m, 0);
   r = g_dbus_connection_send_message_with_reply_sync (c,
                                                       m,
+                                                      G_DBUS_SEND_MESSAGE_FLAGS_NONE,
                                                       -1,
                                                       &data.serial,
                                                       NULL, /* GCancellable */
@@ -706,6 +718,7 @@ test_connection_flush (void)
                                                           "SomeSignal",
                                                           "/org/gtk/GDBus/FlushObject",
                                                           NULL,
+                                                          G_DBUS_SIGNAL_FLAGS_NONE,
                                                           test_connection_flush_signal_handler,
                                                           NULL,
                                                           NULL);
