@@ -1,11 +1,15 @@
 #include <stdlib.h>
 #include <locale.h>
 #include <libintl.h>
-#include <gio.h>
+#include <gio/gio.h>
 #include <gstdio.h>
 #define G_SETTINGS_ENABLE_BACKEND
 #include <gio/gsettingsbackend.h>
 
+#include "testenum.h"
+
+static gboolean backend_set;
+
 /* These tests rely on the schemas in org.gtk.test.gschema.xml
  * to be compiled and installed in the same directory.
  */
@@ -21,8 +25,12 @@ test_basic (void)
 
   settings = g_settings_new ("org.gtk.test");
 
+  g_object_get (settings, "schema", &str, NULL);
+  g_assert_cmpstr (str, ==, "org.gtk.test");
+
   g_settings_get (settings, "greeting", "s", &str);
   g_assert_cmpstr (str, ==, "Hello, earthlings");
+  g_free (str);
 
   g_settings_set (settings, "greeting", "s", "goodbye world");
   g_settings_get (settings, "greeting", "s", &str);
@@ -30,20 +38,28 @@ test_basic (void)
   g_free (str);
   str = NULL;
 
-  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+  if (!backend_set)
     {
-      settings = g_settings_new ("org.gtk.test");
-      g_settings_set (settings, "greeting", "i", 555);
-      abort ();
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        {
+          settings = g_settings_new ("org.gtk.test");
+          g_settings_set (settings, "greeting", "i", 555);
+          abort ();
+        }
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*g_settings_type_check*");
     }
-  g_test_trap_assert_failed ();
-  g_test_trap_assert_stderr ("*correct_type*");
 
   g_settings_get (settings, "greeting", "s", &str);
   g_assert_cmpstr (str, ==, "goodbye world");
   g_free (str);
   str = NULL;
 
+  g_settings_reset (settings, "greeting");
+  str = g_settings_get_string (settings, "greeting");
+  g_assert_cmpstr (str, ==, "Hello, earthlings");
+  g_free (str);
+
   g_settings_set (settings, "greeting", "s", "this is the end");
   g_object_unref (settings);
 }
@@ -115,12 +131,42 @@ test_wrong_type (void)
 
       settings = g_settings_new ("org.gtk.test");
 
-      g_settings_set (settings, "greetings", "o", "/a/path");
+      g_settings_set (settings, "greeting", "o", "/a/path");
     }
   g_test_trap_assert_failed ();
   g_test_trap_assert_stderr ("*CRITICAL*");
 }
 
+/* Check errors with explicit paths */
+static void
+test_wrong_path (void)
+{
+  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+    {
+      GSettings *settings;
+
+      settings = g_settings_new_with_path ("org.gtk.test", "/wrong-path/");
+    }
+
+  g_test_trap_assert_failed ();
+  g_test_trap_assert_stderr ("*but path * specified by schema*");
+}
+
+static void
+test_no_path (void)
+{
+  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+    {
+      GSettings *settings;
+
+      settings = g_settings_new ("org.gtk.test.no-path");
+    }
+
+  g_test_trap_assert_failed ();
+  g_test_trap_assert_stderr ("*attempting to create schema * without a path**");
+}
+
+
 /* Check that we can successfully read and set the full
  * range of all basic types
  */
@@ -532,8 +578,6 @@ test_atomic (void)
   g_object_unref (settings);
 }
 
-#ifndef G_OS_WIN32
-
 /* On Windows the interaction between the C library locale and libintl
  * (from GNU gettext) is not like on POSIX, so just skip these tests
  * for now.
@@ -633,19 +677,23 @@ test_l10n_context (void)
   g_free (locale);
 }
 
-#endif
-
 enum
 {
   PROP_0,
   PROP_BOOL,
+  PROP_BYTE,
+  PROP_INT16,
+  PROP_UINT16,
   PROP_INT,
+  PROP_UINT,
   PROP_INT64,
   PROP_UINT64,
   PROP_DOUBLE,
   PROP_STRING,
   PROP_NO_READ,
-  PROP_NO_WRITE
+  PROP_NO_WRITE,
+  PROP_STRV,
+  PROP_ENUM
 };
 
 typedef struct
@@ -653,13 +701,19 @@ typedef struct
   GObject parent_instance;
 
   gboolean bool_prop;
+  gchar byte_prop;
+  gint int16_prop;
+  guint16 uint16_prop;
   gint int_prop;
+  guint uint_prop;
   gint64 int64_prop;
   guint64 uint64_prop;
   gdouble double_prop;
   gchar *string_prop;
   gchar *no_read_prop;
   gchar *no_write_prop;
+  gchar **strv_prop;
+  guint enum_prop;
 } TestObject;
 
 typedef struct
@@ -678,6 +732,7 @@ static void
 test_object_finalize (GObject *object)
 {
   TestObject *testo = (TestObject*)object;
+  g_strfreev (testo->strv_prop);
   g_free (testo->string_prop);
   G_OBJECT_CLASS (test_object_parent_class)->finalize (object);
 }
@@ -695,9 +750,21 @@ test_object_get_property (GObject    *object,
     case PROP_BOOL:
       g_value_set_boolean (value, test_object->bool_prop);
       break;
+    case PROP_BYTE:
+      g_value_set_char (value, test_object->byte_prop);
+      break;
+    case PROP_UINT16:
+      g_value_set_uint (value, test_object->uint16_prop);
+      break;
+    case PROP_INT16:
+      g_value_set_int (value, test_object->int16_prop);
+      break;
     case PROP_INT:
       g_value_set_int (value, test_object->int_prop);
       break;
+    case PROP_UINT:
+      g_value_set_uint (value, test_object->uint_prop);
+      break;
     case PROP_INT64:
       g_value_set_int64 (value, test_object->int64_prop);
       break;
@@ -713,6 +780,12 @@ test_object_get_property (GObject    *object,
     case PROP_NO_WRITE:
       g_value_set_string (value, test_object->no_write_prop);
       break;
+    case PROP_STRV:
+      g_value_set_boxed (value, test_object->strv_prop);
+      break;
+    case PROP_ENUM:
+      g_value_set_enum (value, test_object->enum_prop);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -732,9 +805,21 @@ test_object_set_property (GObject      *object,
     case PROP_BOOL:
       test_object->bool_prop = g_value_get_boolean (value);
       break;
+    case PROP_BYTE:
+      test_object->byte_prop = g_value_get_char (value);
+      break;
+    case PROP_INT16:
+      test_object->int16_prop = g_value_get_int (value);
+      break;
+    case PROP_UINT16:
+      test_object->uint16_prop = g_value_get_uint (value);
+      break;
     case PROP_INT:
       test_object->int_prop = g_value_get_int (value);
       break;
+    case PROP_UINT:
+      test_object->uint_prop = g_value_get_uint (value);
+      break;
     case PROP_INT64:
       test_object->int64_prop = g_value_get_int64 (value);
       break;
@@ -752,12 +837,41 @@ test_object_set_property (GObject      *object,
       g_free (test_object->no_read_prop);
       test_object->no_read_prop = g_value_dup_string (value);
       break;
+    case PROP_STRV:
+      g_strfreev (test_object->strv_prop);
+      test_object->strv_prop = g_value_dup_boxed (value);
+      break;
+    case PROP_ENUM:
+      test_object->enum_prop = g_value_get_enum (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
+static GType
+test_enum_get_type (void)
+{
+  static volatile gsize define_type_id = 0;
+
+  if (g_once_init_enter (&define_type_id))
+    {
+      static const GEnumValue values[] = {
+        { TEST_ENUM_FOO, "TEST_ENUM_FOO", "foo" },
+        { TEST_ENUM_BAR, "TEST_ENUM_BAR", "bar" },
+        { TEST_ENUM_BAZ, "TEST_ENUM_BAZ", "baz" },
+        { TEST_ENUM_QUUX, "TEST_ENUM_QUUX", "quux" },
+        { 0, NULL, NULL }
+      };
+
+      GType type_id = g_enum_register_static ("TestEnum", values);
+      g_once_init_leave (&define_type_id, type_id);
+    }
+
+  return define_type_id;
+}
+
 static void
 test_object_class_init (TestObjectClass *class)
 {
@@ -769,8 +883,16 @@ test_object_class_init (TestObjectClass *class)
 
   g_object_class_install_property (gobject_class, PROP_BOOL,
     g_param_spec_boolean ("bool", "", "", FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_BYTE,
+    g_param_spec_char ("byte", "", "", G_MININT8, G_MAXINT8, 0, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_INT16,
+    g_param_spec_int ("int16", "", "", -G_MAXINT16, G_MAXINT16, 0, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_UINT16,
+    g_param_spec_uint ("uint16", "", "", 0, G_MAXUINT16, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_INT,
-    g_param_spec_int ("int", "", "", -G_MAXINT, G_MAXINT, 0, G_PARAM_READWRITE));
+    g_param_spec_int ("int", "", "", G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_UINT,
+    g_param_spec_uint ("uint", "", "", 0, G_MAXUINT, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_INT64,
     g_param_spec_int64 ("int64", "", "", G_MININT64, G_MAXINT64, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_UINT64,
@@ -783,6 +905,10 @@ test_object_class_init (TestObjectClass *class)
     g_param_spec_string ("no-write", "", "", NULL, G_PARAM_READABLE));
   g_object_class_install_property (gobject_class, PROP_NO_READ,
     g_param_spec_string ("no-read", "", "", NULL, G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class, PROP_STRV,
+    g_param_spec_boxed ("strv", "", "", G_TYPE_STRV, G_PARAM_READWRITE));
+  g_object_class_install_property (gobject_class, PROP_ENUM,
+    g_param_spec_enum ("enum", "", "", test_enum_get_type (), TEST_ENUM_FOO, G_PARAM_READWRITE));
 }
 
 static TestObject *
@@ -801,11 +927,16 @@ test_simple_binding (void)
   TestObject *obj;
   GSettings *settings;
   gboolean b;
+  gchar y;
   gint i;
+  gint16 n;
+  guint16 q;
   gint64 i64;
   guint64 u64;
   gdouble d;
   gchar *s;
+  GVariant *value;
+  gchar **strv;
 
   settings = g_settings_new ("org.gtk.test.binding");
   obj = test_object_new ();
@@ -816,35 +947,77 @@ test_simple_binding (void)
   g_assert_cmpint (g_settings_get_boolean (settings, "bool"), ==, TRUE);
 
   g_settings_set_boolean (settings, "bool", FALSE);
+  b = TRUE;
   g_object_get (obj, "bool", &b, NULL);
   g_assert_cmpint (b, ==, FALSE);
 
+  g_settings_bind (settings, "byte", obj, "byte", G_SETTINGS_BIND_DEFAULT);
+
+  g_object_set (obj, "byte", 123, NULL);
+  y = 'c';
+  g_settings_get (settings, "byte", "y", &y);
+  g_assert_cmpint (y, ==, 123);
+
+  g_settings_set (settings, "byte", "y", 54);
+  y = 'c';
+  g_object_get (obj, "byte", &y, NULL);
+  g_assert_cmpint (y, ==, 54);
+
+  g_settings_bind (settings, "int16", obj, "int16", G_SETTINGS_BIND_DEFAULT);
+
+  g_object_set (obj, "int16", 1234, NULL);
+  n = 4321;
+  g_settings_get (settings, "int16", "n", &n);
+  g_assert_cmpint (n, ==, 1234);
+
+  g_settings_set (settings, "int16", "n", 4321);
+  n = 1111;
+  g_object_get (obj, "int16", &n, NULL);
+  g_assert_cmpint (n, ==, 4321);
+
+  g_settings_bind (settings, "uint16", obj, "uint16", G_SETTINGS_BIND_DEFAULT);
+
+  g_object_set (obj, "uint16", (guint16) G_MAXUINT16, NULL);
+  q = 1111;
+  g_settings_get (settings, "uint16", "q", &q);
+  g_assert_cmpuint (q, ==, G_MAXUINT16);
+
+  g_settings_set (settings, "uint16", "q", (guint16) G_MAXINT16);
+  q = 1111;
+  g_object_get (obj, "uint16", &q, NULL);
+  g_assert_cmpuint (q, ==, (guint16) G_MAXINT16);
+
   g_settings_bind (settings, "int", obj, "int", G_SETTINGS_BIND_DEFAULT);
 
   g_object_set (obj, "int", 12345, NULL);
   g_assert_cmpint (g_settings_get_int (settings, "int"), ==, 12345);
 
   g_settings_set_int (settings, "int", 54321);
+  i = 1111;
   g_object_get (obj, "int", &i, NULL);
   g_assert_cmpint (i, ==, 54321);
 
   g_settings_bind (settings, "int64", obj, "int64", G_SETTINGS_BIND_DEFAULT);
 
   g_object_set (obj, "int64", (gint64) G_MAXINT64, NULL);
+  i64 = 1111;
   g_settings_get (settings, "int64", "x", &i64);
   g_assert_cmpint (i64, ==, G_MAXINT64);
 
   g_settings_set (settings, "int64", "x", (gint64) G_MININT64);
+  i64 = 1111;
   g_object_get (obj, "int64", &i64, NULL);
   g_assert_cmpint (i64, ==, G_MININT64);
 
   g_settings_bind (settings, "uint64", obj, "uint64", G_SETTINGS_BIND_DEFAULT);
 
   g_object_set (obj, "uint64", (guint64) G_MAXUINT64, NULL);
+  u64 = 1111;
   g_settings_get (settings, "uint64", "t", &u64);
   g_assert_cmpuint (u64, ==, G_MAXUINT64);
 
   g_settings_set (settings, "uint64", "t", (guint64) G_MAXINT64);
+  u64 = 1111;
   g_object_get (obj, "uint64", &u64, NULL);
   g_assert_cmpuint (u64, ==, (guint64) G_MAXINT64);
 
@@ -860,12 +1033,20 @@ test_simple_binding (void)
   g_assert_cmpstr (s, ==, "bla bla");
   g_free (s);
 
+  g_settings_bind (settings, "chararray", obj, "string", G_SETTINGS_BIND_DEFAULT);
+
+  g_object_set (obj, "string", "non-unicode:\315", NULL);
+  value = g_settings_get_value (settings, "chararray");
+  g_assert_cmpstr (g_variant_get_bytestring (value), ==, "non-unicode:\315");
+  g_variant_unref (value);
+
   g_settings_bind (settings, "double", obj, "double", G_SETTINGS_BIND_DEFAULT);
 
   g_object_set (obj, "double", G_MAXFLOAT, NULL);
   g_assert_cmpfloat (g_settings_get_double (settings, "double"), ==, G_MAXFLOAT);
 
   g_settings_set_double (settings, "double", G_MINFLOAT);
+  d = 1.0;
   g_object_get (obj, "double", &d, NULL);
   g_assert_cmpfloat (d, ==, G_MINFLOAT);
 
@@ -873,8 +1054,96 @@ test_simple_binding (void)
   g_assert_cmpfloat (g_settings_get_double (settings, "double"), ==, G_MAXDOUBLE);
 
   g_settings_set_double (settings, "double", -G_MINDOUBLE);
+  d = 1.0;
   g_object_get (obj, "double", &d, NULL);
   g_assert_cmpfloat (d, ==, -G_MINDOUBLE);
+
+  strv = g_strsplit ("plastic bag,middle class,polyethylene", ",", 0);
+  g_settings_bind (settings, "strv", obj, "strv", G_SETTINGS_BIND_DEFAULT);
+  g_object_set (obj, "strv", strv, NULL);
+  g_strfreev (strv);
+  strv = g_settings_get_strv (settings, "strv");
+  s = g_strjoinv (",", strv);
+  g_assert_cmpstr (s, ==, "plastic bag,middle class,polyethylene");
+  g_strfreev (strv);
+  g_free (s);
+  strv = g_strsplit ("decaffeinate,unleaded,keep all surfaces clean", ",", 0);
+  g_settings_set_strv (settings, "strv", (const gchar **) strv);
+  g_strfreev (strv);
+  g_object_get (obj, "strv", &strv, NULL);
+  s = g_strjoinv (",", strv);
+  g_assert_cmpstr (s, ==, "decaffeinate,unleaded,keep all surfaces clean");
+  g_strfreev (strv);
+  g_free (s);
+
+  g_settings_bind (settings, "enum", obj, "enum", G_SETTINGS_BIND_DEFAULT);
+  g_object_set (obj, "enum", TEST_ENUM_BAZ, NULL);
+  s = g_settings_get_string (settings, "enum");
+  g_assert_cmpstr (s, ==, "baz");
+  g_free (s);
+  g_assert_cmpint (g_settings_get_enum (settings, "enum"), ==, TEST_ENUM_BAZ);
+
+  g_settings_set_enum (settings, "enum", TEST_ENUM_QUUX);
+  i = 230;
+  g_object_get (obj, "enum", &i, NULL);
+  g_assert_cmpint (i, ==, TEST_ENUM_QUUX);
+
+  g_settings_set_string (settings, "enum", "baz");
+  i = 230;
+  g_object_get (obj, "enum", &i, NULL);
+  g_assert_cmpint (i, ==, TEST_ENUM_BAZ);
+
+  g_object_unref (obj);
+  g_object_unref (settings);
+}
+
+static void
+test_unbind (void)
+{
+  TestObject *obj;
+  GSettings *settings;
+
+  settings = g_settings_new ("org.gtk.test.binding");
+  obj = test_object_new ();
+
+  g_settings_bind (settings, "int", obj, "int", G_SETTINGS_BIND_DEFAULT);
+
+  g_object_set (obj, "int", 12345, NULL);
+  g_assert_cmpint (g_settings_get_int (settings, "int"), ==, 12345);
+
+  g_settings_unbind (obj, "int");
+
+  g_object_set (obj, "int", 54321, NULL);
+  g_assert_cmpint (g_settings_get_int (settings, "int"), ==, 12345);
+
+  g_object_unref (obj);
+  g_object_unref (settings);
+}
+
+static void
+test_bind_writable (void)
+{
+  TestObject *obj;
+  GSettings *settings;
+  gboolean b;
+
+  settings = g_settings_new ("org.gtk.test.binding");
+  obj = test_object_new ();
+
+  g_object_set (obj, "bool", FALSE, NULL);
+
+  g_settings_bind_writable (settings, "int", obj, "bool", FALSE);
+
+  g_object_get (obj, "bool", &b, NULL);
+  g_assert (b);
+
+  g_settings_unbind (obj, "bool");
+
+  g_settings_bind_writable (settings, "int", obj, "bool", TRUE);
+
+  g_object_get (obj, "bool", &b, NULL);
+  g_assert (!b);
+
   g_object_unref (obj);
   g_object_unref (settings);
 }
@@ -1002,6 +1271,7 @@ test_custom_binding (void)
   g_object_set (obj, "bool", TRUE, NULL);
   s = g_settings_get_string (settings, "string");
   g_assert_cmpstr (s, ==, "true");
+  g_free (s);
 
   g_object_unref (obj);
   g_object_unref (settings);
@@ -1117,22 +1387,23 @@ test_no_write_binding (void)
 static void
 test_keyfile (void)
 {
+  GSettingsBackend *kf_backend;
   GSettings *settings;
   GKeyFile *keyfile;
   gchar *str;
 
   g_remove ("gsettings.store");
 
-  g_settings_backend_setup_keyfile ("blah", "gsettings.store");
-
-  settings = g_settings_new_with_context ("org.gtk.test", "blah");
+  kf_backend = g_keyfile_settings_backend_new ("gsettings.store", "/", "root");
+  settings = g_settings_new_with_backend ("org.gtk.test", kf_backend);
+  g_object_unref (kf_backend);
 
   g_settings_set (settings, "greeting", "s", "see if this works");
 
   keyfile = g_key_file_new ();
   g_assert (g_key_file_load_from_file (keyfile, "gsettings.store", 0, NULL));
 
-  str = g_key_file_get_string (keyfile, "/tests/", "greeting", NULL);
+  str = g_key_file_get_string (keyfile, "tests", "greeting", NULL);
   g_assert_cmpstr (str, ==, "'see if this works'");
 
   g_free (str);
@@ -1169,44 +1440,441 @@ test_child_schema (void)
   g_object_unref (settings);
 }
 
+static gboolean
+glib_translations_work (void)
+{
+  gchar *locale;
+  gchar *orig = "Unnamed";
+  gchar *str;
+
+  locale = g_strdup (setlocale (LC_MESSAGES, NULL));
+  setlocale (LC_MESSAGES, "de");
+  str = dgettext ("glib20", orig);
+  setlocale (LC_MESSAGES, locale);
+  g_free (locale);
+
+  return str != orig;
+}
+
+#include "../strinfo.c"
+
+static void
+test_strinfo (void)
+{
+  /*  "foo" has a value of 1
+   *  "bar" has a value of 2
+   *  "baz" is an alias for "bar"
+   */
+  gchar array[] =
+    "\1\0\0\0"      "\xff""foo"     "\0\0\0\xff"    "\2\0\0\0"
+    "\xff" "bar"    "\0\0\0\xff"    "\3\0\0\0"      "\xfe""baz"
+    "\0\0\0\xff";
+  const guint32 *strinfo = (guint32 *) array;
+  guint length = sizeof array / 4;
+  guint result;
+
+  {
+    /* build it and compare */
+    GString *builder;
+
+    builder = g_string_new (NULL);
+    strinfo_builder_append_item (builder, "foo", 1);
+    strinfo_builder_append_item (builder, "bar", 2);
+    g_assert (strinfo_builder_append_alias (builder, "baz", "bar"));
+    g_assert_cmpint (builder->len % 4, ==, 0);
+    g_assert_cmpint (builder->len / 4, ==, length);
+    g_assert (memcmp (builder->str, strinfo, length * 4) == 0);
+    g_string_free (builder, TRUE);
+  }
+
+  g_assert_cmpstr (strinfo_string_from_alias (strinfo, length, "foo"),
+                   ==, NULL);
+  g_assert_cmpstr (strinfo_string_from_alias (strinfo, length, "bar"),
+                   ==, NULL);
+  g_assert_cmpstr (strinfo_string_from_alias (strinfo, length, "baz"),
+                   ==, "bar");
+  g_assert_cmpstr (strinfo_string_from_alias (strinfo, length, "quux"),
+                   ==, NULL);
+
+  g_assert (strinfo_enum_from_string (strinfo, length, "foo", &result));
+  g_assert_cmpint (result, ==, 1);
+  g_assert (strinfo_enum_from_string (strinfo, length, "bar", &result));
+  g_assert_cmpint (result, ==, 2);
+  g_assert (!strinfo_enum_from_string (strinfo, length, "baz", &result));
+  g_assert (!strinfo_enum_from_string (strinfo, length, "quux", &result));
+
+  g_assert_cmpstr (strinfo_string_from_enum (strinfo, length, 0), ==, NULL);
+  g_assert_cmpstr (strinfo_string_from_enum (strinfo, length, 1), ==, "foo");
+  g_assert_cmpstr (strinfo_string_from_enum (strinfo, length, 2), ==, "bar");
+  g_assert_cmpstr (strinfo_string_from_enum (strinfo, length, 3), ==, NULL);
+
+  g_assert (strinfo_is_string_valid (strinfo, length, "foo"));
+  g_assert (strinfo_is_string_valid (strinfo, length, "bar"));
+  g_assert (!strinfo_is_string_valid (strinfo, length, "baz"));
+  g_assert (!strinfo_is_string_valid (strinfo, length, "quux"));
+}
+
+static void
+test_enums (void)
+{
+  GSettings *settings, *direct;
+  gchar *str;
+
+  settings = g_settings_new ("org.gtk.test.enums");
+  direct = g_settings_new ("org.gtk.test.enums.direct");
+
+  if (!backend_set)
+    {
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_get_enum (direct, "test");
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*not associated with an enum*");
+
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_set_enum (settings, "test", 42);
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*invalid enum value 42*");
+
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_set_string (settings, "test", "qux");
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*g_settings_range_check*");
+
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_get_flags (settings, "test");
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*not associated with a flags*");
+    }
+
+  str = g_settings_get_string (settings, "test");
+  g_assert_cmpstr (str, ==, "bar");
+  g_free (str);
+
+  g_settings_set_enum (settings, "test", TEST_ENUM_FOO);
+
+  str = g_settings_get_string (settings, "test");
+  g_assert_cmpstr (str, ==, "foo");
+  g_free (str);
+
+  g_assert_cmpint (g_settings_get_enum (settings, "test"), ==, TEST_ENUM_FOO);
+
+  g_settings_set_string (direct, "test", "qux");
+
+  str = g_settings_get_string (direct, "test");
+  g_assert_cmpstr (str, ==, "qux");
+  g_free (str);
+
+  str = g_settings_get_string (settings, "test");
+  g_assert_cmpstr (str, ==, "quux");
+  g_free (str);
+
+  g_assert_cmpint (g_settings_get_enum (settings, "test"), ==, TEST_ENUM_QUUX);
+}
+
+static void
+test_flags (void)
+{
+  GSettings *settings, *direct;
+  gchar **strv;
+  gchar *str;
+
+  settings = g_settings_new ("org.gtk.test.enums");
+  direct = g_settings_new ("org.gtk.test.enums.direct");
+
+  if (!backend_set)
+    {
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_get_flags (direct, "test");
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*not associated with a flags*");
+
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_set_flags (settings, "f-test", 0x42);
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*invalid flags value 0x00000042*");
+
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_set_strv (settings, "f-test",
+                             (const gchar **) g_strsplit ("rock", ",", 0));
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*g_settings_range_check*");
+
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_get_enum (settings, "f-test");
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*not associated with an enum*");
+    }
+
+  strv = g_settings_get_strv (settings, "f-test");
+  str = g_strjoinv (",", strv);
+  g_assert_cmpstr (str, ==, "");
+  g_strfreev (strv);
+  g_free (str);
+
+  g_settings_set_flags (settings, "f-test",
+                        TEST_FLAGS_WALKING | TEST_FLAGS_TALKING);
+
+  strv = g_settings_get_strv (settings, "f-test");
+  str = g_strjoinv (",", strv);
+  g_assert_cmpstr (str, ==, "talking,walking");
+  g_strfreev (strv);
+  g_free (str);
+
+  g_assert_cmpint (g_settings_get_flags (settings, "f-test"), ==,
+                   TEST_FLAGS_WALKING | TEST_FLAGS_TALKING);
+
+  strv = g_strsplit ("speaking,laughing", ",", 0);
+  g_settings_set_strv (direct, "f-test", (const gchar **) strv);
+  g_strfreev (strv);
+
+  strv = g_settings_get_strv (direct, "f-test");
+  str = g_strjoinv (",", strv);
+  g_assert_cmpstr (str, ==, "speaking,laughing");
+  g_strfreev (strv);
+  g_free (str);
+
+  strv = g_settings_get_strv (settings, "f-test");
+  str = g_strjoinv (",", strv);
+  g_assert_cmpstr (str, ==, "talking,laughing");
+  g_strfreev (strv);
+  g_free (str);
+
+  g_assert_cmpint (g_settings_get_flags (settings, "f-test"), ==,
+                   TEST_FLAGS_TALKING | TEST_FLAGS_LAUGHING);
+}
+
+static void
+test_range (void)
+{
+  GSettings *settings, *direct;
+
+  settings = g_settings_new ("org.gtk.test.range");
+  direct = g_settings_new ("org.gtk.test.range.direct");
+
+  if (!backend_set)
+    {
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_set_int (settings, "val", 45);
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*g_settings_range_check*");
+
+      if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDERR))
+        g_settings_set_int (settings, "val", 1);
+      g_test_trap_assert_failed ();
+      g_test_trap_assert_stderr ("*g_settings_range_check*");
+    }
+
+  g_assert_cmpint (g_settings_get_int (settings, "val"), ==, 33);
+  g_settings_set_int (direct, "val", 22);
+  g_assert_cmpint (g_settings_get_int (direct, "val"), ==, 22);
+  g_assert_cmpint (g_settings_get_int (settings, "val"), ==, 22);
+  g_settings_set_int (direct, "val", 45);
+  g_assert_cmpint (g_settings_get_int (direct, "val"), ==, 45);
+  g_assert_cmpint (g_settings_get_int (settings, "val"), ==, 33);
+  g_settings_set_int (direct, "val", 1);
+  g_assert_cmpint (g_settings_get_int (direct, "val"), ==, 1);
+  g_assert_cmpint (g_settings_get_int (settings, "val"), ==, 33);
+}
+
+static gboolean
+strv_has_string (const gchar **haystack,
+                 const gchar  *needle)
+{
+  guint n;
+
+  for (n = 0; haystack != NULL && haystack[n] != NULL; n++)
+    {
+      if (g_strcmp0 (haystack[n], needle) == 0)
+        return TRUE;
+    }
+  return FALSE;
+}
+
+static gboolean
+strv_set_equal (const gchar **strv, ...)
+{
+  gint count;
+  va_list list;
+  const gchar *str;
+  gboolean res;
+
+  res = TRUE;
+  count = 0;
+  va_start (list, strv);
+  while (1)
+    {
+      str = va_arg (list, const gchar *);
+      if (str == NULL)
+        break;
+      if (!strv_has_string (strv, str))
+        {
+          res = FALSE;
+          break;
+        }
+      count++;
+    }
+  va_end (list);
+
+  if (res)
+    res = g_strv_length ((gchar**)strv) == count;
+
+  return res;
+}
+
+static void
+test_list_items (void)
+{
+  GSettings *settings;
+  const gchar **items;
+
+  settings = g_settings_new ("org.gtk.test");
+  items = g_settings_list_items (settings);
+
+  g_assert (strv_set_equal (items, "greeting", "farewell", "basic-types/", "complex-types/", "localized/", NULL));
+
+  g_free (items);
+
+  g_object_unref (settings);
+}
+
+static gboolean
+map_func (GVariant *value,
+          gpointer *result,
+          gpointer  user_data)
+{
+  gint *state = user_data;
+  gint v;
+
+  if (value)
+    v = g_variant_get_int32 (value);
+  else
+    v = -1;
+
+  if (*state == 0)
+    {
+      g_assert_cmpint (v, ==, 1);
+      (*state)++;
+      return FALSE;
+    }
+  else if (*state == 1)
+    {
+      g_assert_cmpint (v, ==, 0);
+      (*state)++;
+      return FALSE;
+    }
+  else
+    {
+      g_assert (value == NULL);
+      *result = g_variant_new_int32 (5);
+      return TRUE;
+    }
+}
+
+static void
+test_get_mapped (void)
+{
+  GSettings *settings;
+  gint state;
+  gpointer p;
+  gint val;
+
+  settings = g_settings_new ("org.gtk.test.mapped");
+  g_settings_set_int (settings, "val", 1);
+
+  state = 0;
+  p = g_settings_get_mapped (settings, "val", map_func, &state);
+  val = g_variant_get_int32 ((GVariant*)p);
+  g_assert_cmpint (val, ==, 5);
+
+  g_variant_unref (p);
+  g_object_unref (settings);
+}
+
 int
 main (int argc, char *argv[])
 {
+  gchar *enums;
+  gint result;
+
   setlocale (LC_ALL, "");
 
+  backend_set = g_getenv ("GSETTINGS_BACKEND") != NULL;
+
   g_setenv ("GSETTINGS_SCHEMA_DIR", ".", TRUE);
-  g_setenv ("GSETTINGS_BACKEND", "memory", TRUE);
+
+  if (!backend_set)
+    g_setenv ("GSETTINGS_BACKEND", "memory", TRUE);
 
   g_type_init ();
   g_test_init (&argc, &argv, NULL);
 
+  g_remove ("org.gtk.test.enums.xml");
+  g_assert (g_spawn_command_line_sync ("../../gobject/glib-mkenums "
+                                       "--template " SRCDIR "/enums.xml.template "
+                                       SRCDIR "/testenum.h",
+                                       &enums, NULL, &result, NULL));
+  g_assert (result == 0);
+  g_assert (g_file_set_contents ("org.gtk.test.enums.xml", enums, -1, NULL));
+  g_free (enums);
+
   g_remove ("gschemas.compiled");
-  g_assert (g_spawn_command_line_sync ("../gschema-compile --targetdir=. " SRCDIR,
-                                       NULL, NULL, NULL, NULL));
+  g_assert (g_spawn_command_line_sync ("../glib-compile-schemas --targetdir=. "
+                                       "--schema-file=org.gtk.test.enums.xml "
+                                       "--schema-file=" SRCDIR "/org.gtk.test.gschema.xml",
+                                       NULL, NULL, &result, NULL));
+  g_assert (result == 0);
 
   g_test_add_func ("/gsettings/basic", test_basic);
-  g_test_add_func ("/gsettings/no-schema", test_no_schema);
-  g_test_add_func ("/gsettings/unknown-key", test_unknown_key);
-  g_test_add_func ("/gsettings/wrong-type", test_wrong_type);
+
+  if (!backend_set)
+    {
+      g_test_add_func ("/gsettings/no-schema", test_no_schema);
+      g_test_add_func ("/gsettings/unknown-key", test_unknown_key);
+      g_test_add_func ("/gsettings/wrong-type", test_wrong_type);
+      g_test_add_func ("/gsettings/wrong-path", test_wrong_path);
+      g_test_add_func ("/gsettings/no-path", test_no_path);
+    }
+
   g_test_add_func ("/gsettings/basic-types", test_basic_types);
   g_test_add_func ("/gsettings/complex-types", test_complex_types);
   g_test_add_func ("/gsettings/changes", test_changes);
-#ifndef G_OS_WIN32
-  g_test_add_func ("/gsettings/l10n", test_l10n);
-  g_test_add_func ("/gsettings/l10n-context", test_l10n_context);
-#endif
+
+  if (glib_translations_work ())
+    {
+      g_test_add_func ("/gsettings/l10n", test_l10n);
+      g_test_add_func ("/gsettings/l10n-context", test_l10n_context);
+    }
+
   g_test_add_func ("/gsettings/delay-apply", test_delay_apply);
   g_test_add_func ("/gsettings/delay-revert", test_delay_revert);
   g_test_add_func ("/gsettings/atomic", test_atomic);
+
   g_test_add_func ("/gsettings/simple-binding", test_simple_binding);
   g_test_add_func ("/gsettings/directional-binding", test_directional_binding);
-  g_test_add_func ("/gsettings/typesafe-binding", test_typesafe_binding);
   g_test_add_func ("/gsettings/custom-binding", test_custom_binding);
   g_test_add_func ("/gsettings/no-change-binding", test_no_change_binding);
-  g_test_add_func ("/gsettings/no-read-binding", test_no_read_binding);
-  g_test_add_func ("/gsettings/no-write-binding", test_no_write_binding);
+  g_test_add_func ("/gsettings/unbinding", test_unbind);
+  g_test_add_func ("/gsettings/writable-binding", test_bind_writable);
+
+  if (!backend_set)
+    {
+      g_test_add_func ("/gsettings/typesafe-binding", test_typesafe_binding);
+      g_test_add_func ("/gsettings/no-read-binding", test_no_read_binding);
+      g_test_add_func ("/gsettings/no-write-binding", test_no_write_binding);
+    }
+
   g_test_add_func ("/gsettings/keyfile", test_keyfile);
   g_test_add_func ("/gsettings/child-schema", test_child_schema);
+  g_test_add_func ("/gsettings/strinfo", test_strinfo);
+  g_test_add_func ("/gsettings/enums", test_enums);
+  g_test_add_func ("/gsettings/flags", test_flags);
+  g_test_add_func ("/gsettings/range", test_range);
+  g_test_add_func ("/gsettings/list-items", test_list_items);
+  g_test_add_func ("/gsettings/mapped", test_get_mapped);
 
-  return g_test_run ();
+  result = g_test_run ();
+
+  g_settings_sync ();
+
+  return result;
 }
