@@ -31,7 +31,26 @@
 #include "glib.h"
 #include "glibintl.h"
 
-#include "galias.h"
+
+/**
+ * SECTION:base64 Base64 Encoding
+ * @short_description: encodes and decodes data in Base64 format
+ *
+ * Base64 is an encoding that allows a sequence of arbitrary bytes to be
+ * encoded as a sequence of printable ASCII characters. For the definition
+ * of Base64, see <ulink url="http://www.ietf.org/rfc/rfc1421.txt">RFC
+ * 1421</ulink> or <ulink url="http://www.ietf.org/rfc/rfc2045.txt">RFC
+ * 2045</ulink>. Base64 is most commonly used as a MIME transfer encoding
+ * for email.
+ *
+ * GLib supports incremental encoding using g_base64_encode_step() and
+ * g_base64_encode_close(). Incremental decoding can be done with
+ * g_base64_decode_step(). To encode or decode data in one go, use
+ * g_base64_encode() or g_base64_decode(). To avoid memory allocation when
+ * decoding, you can use g_base64_decode_inplace().
+ *
+ * Support for Base64 encoding has been added in GLib 2.12.
+ */
 
 static const char base64_alphabet[] =
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -165,14 +184,18 @@ g_base64_encode_step (const guchar *in,
  *
  * Flush the status from a sequence of calls to g_base64_encode_step().
  *
+ * The output buffer must be large enough to fit all the data that will
+ * be written to it. It will need up to 4 bytes, or up to 5 bytes if
+ * line-breaking is enabled.
+ *
  * Return value: The number of bytes of output that was written
  *
  * Since: 2.12
  */
 gsize
 g_base64_encode_close (gboolean  break_lines,
-		       gchar    *out, 
-		       gint     *state, 
+		       gchar    *out,
+		       gint     *state,
 		       gint     *save)
 {
   int c1, c2;
@@ -231,8 +254,7 @@ g_base64_encode (const guchar *data,
   gint state = 0, outlen;
   gint save = 0;
 
-  g_return_val_if_fail (data != NULL, NULL);
-  g_return_val_if_fail (len > 0, NULL);
+  g_return_val_if_fail (data != NULL || len == 0, NULL);
 
   /* We can use a smaller limit here, since we know the saved state is 0,
      +1 is needed for trailing \0, also check for unlikely integer overflow */
@@ -375,8 +397,6 @@ g_base64_decode (const gchar *text,
 
   input_length = strlen (text);
 
-  g_return_val_if_fail (input_length > 1, NULL);
-
   /* We can use a smaller limit here, since we know the saved state is 0,
      +1 used to avoid calling g_malloc0(0), and hence retruning NULL */
   ret = g_malloc0 ((input_length / 4) * 3 + 1);
@@ -415,9 +435,5 @@ g_base64_decode_inplace (gchar *text,
 
   *out_len = g_base64_decode_step (text, input_length, (guchar *) text, &state, &save);
   
-  return text; 
+  return (guchar *) text; 
 }
-
-
-#define __G_BASE64_C__
-#include "galiasdef.c"
